@@ -180,9 +180,3 @@ impl TaskExecutionQueue {
         self.0.len()
     }
 }
-
-#[derive(Default)]
-pub struct Cron {
-    pub is_running: bool,
-    pub scheduler: TaskScheduler,
-}
